//---------------------------------------------------------------------------------
/// \file	MyFunction.cpp
/// \author	Rob Bateman [robthebloke@hotmail.com]
/// \date	2-7-2004
/// \brief	We now want to improve this code a little so that we can make use of
/// 		the MSyntax method of handling selected objects. Basically the default
/// 		maya commands often work upon the selected objects, or they work on
/// 		a list of maya nodes provided as an argument.
///
/// 		We could for example call our command like so :
///
/// 		myFunction "pSphere1" "pSphere2";
///
/// 		without the objects specified, the command by default will just retrieve
/// 		the active selection list.
///
/// 		As you will no doubt see, the MArgDatabase and MSyntax classes cut out
/// 		a great deal of work when attempting this.
///	
//---------------------------------------------------------------------------------

#ifdef WIN32
	#define NT_PLUGIN
#endif

// include any maya headers we need.
#include "MyFunction.h"
#include <maya/MSelectionList.h>
#include <maya/MArgList.h>
#include <maya/MArgDatabase.h>

/// the long and short help flags
const char* g_HelpFlag = "-h";
const char* g_HelpFlagLong = "-help";

/// the text printed when the -h/-help flag is specified
const char* g_HelpText = "I'm a slightly longer description of how to use the function";


//---------------------------------------------------------------------------------
/// \brief	This function must create a new syntax description for the mel command.
/// 		This allows us to simplify the parsing of arguments passed to the function
/// 		call.
/// \note	If in mel you type...
///
/// 			help myFunction
///
///			... it should output the following to the script editor
///
/// 			Synopsis: myFunction [flags]
/// 			Flags:
/// 			   -h -help
/// 			  -sl -selected  on|off
///
/// 		Maya uses the MSyntax object we create to produce this information.
///
MSyntax MyFunction::newSyntax()
{
	MSyntax syn;

	/// add a simple -h flag
	syn.addFlag( g_HelpFlag, g_HelpFlagLong );

	/// specify that our mel command handles objects
	syn.setObjectType(MSyntax::kSelectionList);

	/// if no objects are specified, then tell the command to use the selected
	/// objects.
	syn.useSelectionAsDefault(true);

	return syn;
}

//---------------------------------------------------------------------------------
bool MyFunction::isUndoable() const {
	return false;
}

//---------------------------------------------------------------------------------
/// \brief	this function is called by maya to return a new instance of our mel function
/// \return	the new mel command object
///
void* MyFunction::creator() {
	return new MyFunction;
}

//---------------------------------------------------------------------------------
/// \brief	an internal function to simply add a set of lights to the mesh object
/// 		specified.
/// \param	oMesh	-	the handle to the object to attach the lights to
/// \return	a status code
/// 
MStatus MyFunction::addLightsToMesh(MObject oMesh) {

	MFnPointLight fnLight;
	// attach a function set to the mesh
	MFnMesh fnMesh( oMesh );

	// get the parent transform of the mesh and attach a function set
	MObject oParent = fnMesh.parent(0);
	MFnTransform fnParent( oParent );


	// retrieve the mesh vertex data
	MPointArray points;
	fnMesh.getPoints( points );


	// loop through each point on the mesh
	for(unsigned int i=0; i<points.length(); ++i )
	{
		// create a vector from the current mesh vertex
		MVector V;
		V.x = points[i].x;
		V.y = points[i].y;
		V.z = points[i].z;

		MFnTransform fnX;

		// create a transform for the light instance, specifying the meshes
		// parent transform as the parent for the new transform. Basically
		// we are going to add the light instance as a child of the mesh. This
		// way when we move the mesh, all of the lights will follow...
		//
		fnX.create( fnMesh.parent(0) );

		// set the translation value
		fnX.setTranslation( V, MSpace::kTransform );

		// create a point light and store the parent transform of the light
		MObject oLight = fnLight.create(fnX.object());

		MColor col(1,0,0);
		fnLight.setColor(col);

		// add the light to the transform we created for it. If the last paramater
		// was false, then it would unparent the light from any
		fnX.addChild( oLight , MFnDagNode::kNextPos , true );
	}
	return MS::kSuccess;
}

//---------------------------------------------------------------------------------
/// \brief	This function impliments our very first simple command plugin.
/// \param	args - the argument list that was passes to the command from MEL
/// \return	MS::kSuccess - command succeeded
///			MS::kFailure - command failed (returning this value will cause the
///         	           MEL script that is being run to terminate unless the
///         	           error is caught using a "catch" statement.
///
MStatus MyFunction::doIt( const MArgList& args )
{
	// the return status
	MStatus stat = MS::kSuccess;

	// our flag
	bool doSelected=false;

	// create a parser for the arguments.
	MArgDatabase argData(syntax(),args,&stat);

	// make sure the arguments were parsed OK
	if(stat != MS::kSuccess)
	{
		stat.perror("arguments not specified correctly");
		return stat;
	}

	// if the help flag has been set, simply print the results to the script editor
	if(argData.isFlagSet(g_HelpFlag))
	{
		setResult(g_HelpText);
		return MS::kSuccess;
	}

	// get the current selection list from maya
	MSelectionList selected;
	argData.getObjects(selected);

	// loop through the selected items
	for(unsigned int i=0;i<selected.length();++i)
	{
		// get the MObject for this item
		MObject oObj;
		selected.getDependNode(i,oObj);

		// if we have a mesh selected
		if(oObj.apiType()==MFn::kMesh)
			addLightsToMesh( oObj );
		else
		// if we have a transform of some sort
		if( oObj.hasFn(MFn::kTransform) )
		{
			// check all of it's children to see if a mesh is parented directly underneath
			MFnTransform fn(oObj);
			for(unsigned int j=0;j<fn.childCount();++j)
			{
				// check to see if this child is a mesh
				MObject oChild = fn.child(j);
				if(oChild.apiType()==MFn::kMesh)
					addLightsToMesh( oChild );

			}
		}
	}

	return stat;
}

