//---------------------------------------------------------------------------------
/// \file	MyFunction.h
/// \author	Rob Bateman [robthebloke@hotmail.com]
/// \date	2-7-2004
/// \brief	We now want to improve this code a little so that we can make use of
/// 		the MSyntax method of handling selected objects. Basically the default
/// 		maya commands often work upon the selected objects, or they work on
/// 		a list of maya nodes provided as an argument.
///
/// 		We could for example call our command like so :
///
/// 		myFunction "pSphere1" "pSphere2";
///
/// 		without the objects specified, the command by default will just retrieve
/// 		the active selection list.
///
/// 		As you will no doubt see, the MArgDatabase and MSyntax classes cut out
/// 		a great deal of work when attempting this.
///
//---------------------------------------------------------------------------------

#ifndef __MY_MEL_COMMAND__H__
#define __MY_MEL_COMMAND__H__

	#ifdef WIN32
		#define NT_PLUGIN
		#pragma once
	#endif

	// include any maya headers we need.
	#include <maya/MPxCommand.h>
	#include <maya/MSyntax.h>
	#include <maya/MGlobal.h>
	#include <maya/MFnMesh.h>
	#include <maya/MFnTransform.h>
	#include <maya/MFnPointLight.h>
	#include <maya/MItDependencyNodes.h>
	#include <maya/MPointArray.h>

	class MyFunction
		: public MPxCommand {
	public:

		/// \brief	This function is overidden to tell maya whether this mel command
		/// 		can be undone. To keep things simple, returning false indicates to
		/// 		maya that this command will not undo itself.../
		/// \return	false
		///
		virtual bool isUndoable() const ;

		/// \brief	This function is called when you mel command is executed. The function
		/// 		recieves the list of arguments passed to the command by mel.
		/// \param	args	-	the list of arguments
		/// \param	An error code.
		///
		MStatus doIt( const MArgList& args );


		/// \brief	this function is called by maya to return a new instance of our mel function
		/// \return	the new mel command object
		///
		static void* creator();
		
		/// \brief	This function must create a new syntax description for the mel command.
		/// 		This allows us to simplify the parsing of arguments passed to the function
		/// 		call.
		/// \note	If in mel you type...
		///
		/// 			help myFunction
		///
		///			... it should output the following to the script editor
		///
		/// 			Synopsis: myFunction [flags]
		/// 			Flags:
		/// 			   -h -help
		/// 			  -sl -selected  on|off
		///
		/// 		Maya uses the MSyntax object we create to produce this information.
		///
		static MSyntax newSyntax();

	private:

		/// \brief	an internal function to simply add a set of lights to the mesh object
		/// 		specified.
		/// \param	oMesh	-	the handle to the object to attach the lights to
		/// \return	a status code
		///
		MStatus addLightsToMesh(MObject oMesh);
	};

#endif
